import contextvars
from aiomoqt.types import MOQT_CUR_VERSION

# Define the context variable
moqt_version_context = contextvars.ContextVar('moqt_version', default=MOQT_CUR_VERSION)

def get_moqt_ctx_version() -> int:
    return moqt_version_context.get()

def set_moqt_ctx_version(version: int = MOQT_CUR_VERSION) -> contextvars.Token:
    return moqt_version_context.set(version)

def reset_moqt_ctx_version(token: contextvars.Token) -> None:
    moqt_version_context.reset(token)

def get_major_version(version: int) -> bool:
    if (version & 0x00ff0000):
        return (version & 0x00ff0000) >> 16
    else:
        return (version & 0x0000ffff)
        


#!/usr/bin/env python3
import asyncio
import argparse
import logging

from aioquic.h3.connection import H3_ALPN
from aiomoqt.types import ParamType, MOQTException
from aiomoqt.client import MOQTClientSession
from aiomoqt.protocol import MOQTException
from aiomoqt.messages.subscribe import SubscribeError
from aiomoqt.messages.announce import SubscribeAnnouncesError 
from aiomoqt.utils.logger import *

def parse_args():
    parser = argparse.ArgumentParser(description='MOQT WebTransport Client')
    parser.add_argument('--host', type=str, default='localhost', help='Host to connect to')
    parser.add_argument('--port', type=int, default=4433, help='Port to connect to')
    parser.add_argument('--namespace', type=str, default="live/test", help='Track Namespace')
    parser.add_argument('--trackname', type=str, default="track", help='Track Name')
    parser.add_argument('--endpoint', type=str, default="moq", help='MOQT WT endpoint')
    parser.add_argument('--debug', action='store_true', help='Enable debug output')
    parser.add_argument('--keylogfile', type=str, default=None, help='TLS secrets file')
    return parser.parse_args()


async def main(host: str, port: int, endpoint: str, namespace: str, track_name: str, debug: bool):
    log_level = logging.DEBUG if debug else logging.INFO
    set_log_level(log_level)
    logger = get_logger(__name__)

    client = MOQTClientSession(
        host,
        port,
        endpoint=endpoint,
        keylog_filename=args.keylogfile,
        debug=debug
    )
    logger.info(f"MOQT app: subscribe session connecting: {client}")
    try:
        async with client.connect() as session:
            try: 
                response = await session.client_session_init()

                response = await session.subscribe_announces(
                    namespace_prefix=namespace,
                    parameters={ParamType.AUTHORIZATION_INFO: b"auth-token-123"},
                    wait_response=True
                )
                
                if isinstance(response, SubscribeAnnouncesError):
                    logger.error(f"MOQT app: {response}")
                    raise MOQTException(response.error_code, response.reason)

                sub_response, fetch_response = await session.join(
                    namespace=namespace,
                    track_name=track_name,
                    parameters={
                        ParamType.MAX_CACHE_DURATION: 100,
                        ParamType.AUTHORIZATION_INFO: b"auth-token-123",
                        ParamType.DELIVERY_TIMEOUT: 10,
                    },
                    wait_response=True
                )

                if isinstance(response, SubscribeError):
                    logger.error(f"MOQT app: {response}")
                    raise MOQTException(response.error_code, response.reason)
                # process subscription - publisher will open stream and send data
                await session.async_closed()
                logger.info(f"MOQT app: exiting client session")
            except MOQTException as e:
                logger.error(f"MOQT app: session exception: {e}")
                session.close(e.error_code, e.reason_phrase)
                pass
            except Exception as e:
                logger.error(f"MOQT app: connection failed: {e}")
                pass
    except Exception as e:
        logger.error(f"MOQT app: connection failed: {e}")
        pass
    
    logger.info(f"MOQT app: subscribe session closed: {class_name(client)}")

if __name__ == "__main__":
    try:
        args = parse_args()
        asyncio.run(main(
            host=args.host,
            port=args.port,
            endpoint=args.endpoint,
            namespace=args.namespace,
            track_name=args.trackname,
            debug=args.debug
        ), debug=args.debug)
    
    except KeyboardInterrupt:
        pass


from enum import IntEnum


MOQT_VERSIONS = [0xff000008, 0xff080000, 0xff000009, 0xff090000, 0xff00000a, 0xff0a0000]
MOQT_CUR_VERSION = 0xff000008


class MOQTMessageType(IntEnum):
    """MOQT message type constants."""
    CLIENT_SETUP = 0x40
    SERVER_SETUP = 0x41
    SUBSCRIBE_UPDATE = 0x02
    SUBSCRIBE = 0x03
    SUBSCRIBE_OK = 0x04
    SUBSCRIBE_ERROR = 0x05
    ANNOUNCE = 0x06
    ANNOUNCE_OK = 0x07
    ANNOUNCE_ERROR = 0x08
    UNANNOUNCE = 0x09
    UNSUBSCRIBE = 0x0A
    SUBSCRIBE_DONE = 0x0B
    ANNOUNCE_CANCEL = 0x0C
    TRACK_STATUS_REQUEST = 0x0D
    TRACK_STATUS = 0x0E
    GOAWAY = 0x10
    SUBSCRIBE_ANNOUNCES = 0x11
    SUBSCRIBE_ANNOUNCES_OK = 0x12
    SUBSCRIBE_ANNOUNCES_ERROR = 0x13
    UNSUBSCRIBE_ANNOUNCES = 0x14
    MAX_SUBSCRIBE_ID = 0x15
    FETCH = 0x16
    FETCH_CANCEL = 0x17
    FETCH_OK = 0x18
    FETCH_ERROR = 0x19
    SUBSCRIBES_BLOCKED = 0x1A


class ParamType(IntEnum):
    """Parameter types for MOQT messages."""
    AUTHORIZATION_INFO = 0x02
    DELIVERY_TIMEOUT = 0x03
    MAX_CACHE_DURATION = 0x04
    GREASE_1_PARAM = 0x25
    GREASE_2_PARAM = 0x3D


class SetupParamType(IntEnum):
    """Setup Parameter type constants"""
    CLIENT_ROLE = 0x0  # deprecated - removed in draft-8
    ENDPOINT_PATH = 0x01  # only relevant to raw QUIC connection
    MAX_SUBSCRIBER_ID = 0x02


class SessionCloseCode(IntEnum):
    """Session close error codes."""
    NO_ERROR = 0x0
    INTERNAL_ERROR = 0x01
    UNAUTHORIZED = 0x02
    PROTOCOL_VIOLATION = 0x03
    DUPLICATE_TRACK_ALIAS = 0x04
    PARAMETER_LENGTH_MISMATCH = 0x05
    TOO_MANY_SUBSCRIBES = 0x06
    GOAWAY_TIMEOUT = 0x10
    CONTROL_MESSAGE_TIMEOUT = 0x11
    DATA_STREAM_TIMEOUT = 0x12

class ContentExistsCode(IntEnum):
    """Content Exists Code"""
    NO_CONTENT = 0x0
    EXISTS = 0x01
    
class SubscribeErrorCode(IntEnum):
    """SUBSCRIBE_ERROR error codes."""
    INTERNAL_ERROR = 0x0
    INVALID_RANGE = 0x01
    RETRY_TRACK_ALIAS = 0x02
    TRACK_DOES_NOT_EXIST = 0x03
    UNAUTHORIZED = 0x04
    TIMEOUT = 0x05


class SubscribeDoneCode(IntEnum):
    """SUBSCRIBE_DONE status codes."""
    UNSUBSCRIBED = 0x0
    INTERNAL_ERROR = 0x01
    UNAUTHORIZED = 0x02
    TRACK_ENDED = 0x03
    SUBSCRIPTION_ENDED = 0x04
    GOING_AWAY = 0x05
    EXPIRED = 0x06
    TOO_FAR_BEHIND = 0x07


class TrackStatusCode(IntEnum):
    """TRACK_STATUS status codes."""
    IN_PROGRESS = 0x00
    DOES_NOT_EXIST = 0x01
    NOT_STARTED = 0x02
    FINISHED = 0x03
    RELAY_NO_INFO = 0x04


class FilterType(IntEnum):
    """Subscription filter types."""
    LATEST_OBJECT = 0x02
    ABSOLUTE_START = 0x03
    ABSOLUTE_RANGE = 0x04


class GroupOrder(IntEnum):
    """Group ordering preferences."""
    PUBLISHER_DEFAULT = 0x0
    ASCENDING = 0x01
    DESCENDING = 0x02


class ObjectStatus(IntEnum):
    """Object status codes."""
    NORMAL = 0x0
    DOES_NOT_EXIST = 0x01
    END_OF_GROUP = 0x03
    END_OF_TRACK_AND_GROUP = 0x04
    END_OF_TRACK = 0x05


class ForwardingPreference(IntEnum):
    """Object forwarding preferences."""
    TRACK = 0x0
    SUBGROUP = 0x01
    DATAGRAM = 0x02


class FetchType(IntEnum):
    FETCH = 0x01
    JOINING_FETCH = 0x02


class DataStreamType(IntEnum):
    """Stream type identifiers."""
    SUBGROUP_HEADER = 0x04
    FETCH_HEADER = 0x05


class DatagramType(IntEnum):
    """Datagram type identifiers."""
    OBJECT_DATAGRAM = 0x01
    OBJECT_DATAGRAM_STATUS = 0x02


class MOQTException(Exception):
    def __init__(self, error_code: SessionCloseCode, reason_phrase: str):
        self.error_code = error_code
        self.reason_phrase = reason_phrase
        super().__init__(f"{reason_phrase} ({error_code})")
        
